///////////////////////////
// ADSCF2.h 
// 
// Applied Data Systems
// 
// see ADSCF2.cpp for history information
////////////////////////////
#include <windows.h>


// For BitsyX devices, use this #ifdef, for SA1110 devices comment it out
#define PXA255


#ifdef PXA255
	// Unlike the SA1110, The PXA255 controls PCMCIA timing with several registers
	// Due to complexity, see the PXA255 developer's manual from Intel for more
	// information.  Section 6, pages 57-60
	// MCMEM0 Offset = 0x28
	// MCMEM1 Offset = 0x2C
	// MCATT0 Offset = 0x30
	// MCATT1 Offset = 0x34
	// MCIO0  Offset = 0x38
	// MCIO1  Offset = 0x3C

	#define MCMEM0 0x28
	#define MCMEM1 0x2C
	#define MCATT0 0x30
	#define MCATT1 0x34
	#define MCIO0  0x38
	#define MCIO1  0x3C
	
	#define	MEM_CTRL_REGISTER_ADDR		0x48000000
	#define SA1111_ADDRESS	0x0C000000
	#define GPIO_OFFSET		0x1000
	#define GPIO_BASE		(SA1111_ADDRESS + 0x1000)
	#define PCMCIA_BASE		(SA1111_ADDRESS + 0x1800)
#else
	// SA1110 behavior is affected largely by the MECR register as follows:
	//	MECR
	//	Physical address = 0xA0000014
	//
	//	Bit		Name	Description
	//	------	------	----------------------------------------
	//	4..0	BSIO0	Slot0 I/O space memory clock count
	//  9..5	BSA0	Slot0 attribute space memory clock count
	//	14..10	BSM0	Slot0 memory space memory clock count
	//	15		FAST0	Fast bit for Slot0
	//	20..16	BSIO1	Slot1 I/O space memory clock count
	//  25..21	BSA1	Slot1 attribute space memory clock count
	//	30..26	BSM1	Slot1 memory space memory clock count
	//	31		FAST1	Fast bit for Slot1

	#define	MEM_CTRL_REGISTER_ADDR		0xA0000000
	#define SA1111_ADDRESS	0x18000000
	#define GPIO_OFFSET		0x1000
	#define PCMCIA_BASE		(SA1111_ADDRESS + 0x1800)
#endif


// these constants can be passed to MapAddress to get a base
// virtual address that is then used as the base for Write/ReadData16

// PCMCIA Slot
#define	SLOT0_IO_ADDR	0x20000000 // to 0x23FFFFFF
#define	SLOT0_ATTR_ADDR	0x28000000 // to 0x2BFFFFFF
#define	SLOT0_MEM_ADDR	0x2C000000 // to 0x2FFFFFFF

// Compact Flash Slot
#define	SLOT1_IO_ADDR	0x30000000 // to 0x33FFFFFF
#define	SLOT1_ATTR_ADDR	0x38000000 // to 0x3BFFFFFF
#define	SLOT1_MEM_ADDR	0x3C000000 // to 0x3FFFFFFF

// enums are used instead of constants to help ensure
// library calls do not use values outside of valid ranges

enum SLOT
{
	PCMCIA = 0,
	CF = 1
};

enum BUSPOWER
{
	BITSY_OFF = 0,
	BITSY_3V = 1,
	BITSY_5V = 2,
	BITSYPLUS_OFF = 0,
	BITSYPLUS_3V = 1,
	BITSYPLUS_5V = 2,
	AGC_OFF = 0,
	AGC_3V = 1,
	AGC_5V = 2
};


LPVOID	MapAddress(DWORD baseAddress);
/*void	BusPower(SLOT slot, BUSPOWER powerConstant);
void	BusReset(SLOT slot, ULONG pulseDuration);
void	SetAttributeSpeed(SLOT slot, BYTE newSpeed);
void	SetMECRValue(SLOT slot, WORD newValue);
void	SetIOSpeed(SLOT slot, BYTE newSpeed);
void	SetMemorySpeed(SLOT slot, BYTE newSpeed);
void	SetFASTMode(SLOT slot, BOOL fastOn);*/
void	*WriteData16(LPVOID baseAddress, DWORD offset, USHORT *data, UINT writes);
void	*ReadData16(LPVOID baseAddress, DWORD offset, USHORT *data, UINT reads);
void	ReleaseAddress(LPVOID address);
void	FreeResources();
