///////////////////////////////////////////////////
// Main.cpp
// 720020-11xx2
//
// Applied Data Systems
//
// See setmac.h for a description and history
// 
///////////////////////////////////////////////////

#include <windows.h>
#include <winbase.h>
#include "ADSCF2.h"
#include "smchw.h"
#include "mapio.h"
#include "setmac.h"


LPVOID pMem = NULL;
ULONG index = 0;

/************************* WinMain Method *************************/
int WINAPI WinMain(
    HINSTANCE   hInst,
    HINSTANCE   hInstPrev,
    LPTSTR      lpszCmdLine,
    int         nCmdShow)
{
	USHORT bank;
	UCHAR cIsCorrect;
	UCHAR cDoWrite;
	USHORT tmp;
	USHORT *pStartMem;
	DWORD localAddr;
	DWORD baseAddr = SLOT1_ATTR_ADDR + SMC_BASE;
	WCHAR wszFilename [BUF_LEN];
	DWORD nFilenameLen;
	WCHAR *pwcTemp;
	FILE *fp;
	BOOL bAssignMACFromFile;
	BOOL bAssignMACFromCmdLine;
	UCHAR buf [128];
	WCHAR wbuf [] = L"xx-xx-xx-xx-xx-xx";
	UCHAR szNextMac[] = "xx-xx-xx-xx-xx-xx";
	UCHAR NextMac [6];
	UCHAR temp[6];
	
	// Map SMSC91C111 I/O space to virtual memory (PCMCIA slot 1)
	index = (baseAddr%PAGE_SIZE)/sizeof(USHORT);
	localAddr = (baseAddr/PAGE_SIZE)*PAGE_SIZE;

	pStartMem = (USHORT *) MapPhysicalAddr((void *)localAddr, PAGE_SIZE);
	pMem = pStartMem + index; 

	if (pMem == NULL) {
		RETAILMSG(1,(L" ERROR : WinMain() - pMem == NULL\r\n"));
		return -1;
	}

	// Try writing and reading back from the bank select register
	// to make sure that we can access it.
	WriteReg(BANKSEL_REG, 0);
	bank = ReadReg(BANKSEL_REG);
	if ((bank & 0xff00) != 0x3300)	// Upper byte should be hardwired to 0x33
    {								// otherwise, we are not reading BANKSEL_REG
      	RETAILMSG(1,(L" ERROR : WinMain() - bank select 0x%x \r\n", bank & 0xff00));
      	return -1;
    }
	
	// Reset the ethernet chip
   	SelectBank(BANK1);
	WriteReg(CONFIG_REG, 0xa0b1);
	SMCChipReset();
  	
	// Read and display the current MAC address
	ReadMACAddress(temp);
 	printf("Current MAC Address = %02x-%02x-%02x-%02x-%02x-%02x\r\n",
		       temp[0],temp[1],temp[2], temp[3],temp[4],temp[5]);
	
	// Check if a MAC address was given on the command line	
	if(wcslen(lpszCmdLine) != 0) {
		WideCharToMultiByte(CP_ACP, WC_COMPOSITECHECK | WC_SEPCHARS, lpszCmdLine, -1, (char *)buf, sizeof(buf), NULL, NULL);
		if (!GetMACFromString(buf, NextMac)) {
			printf("ERROR : WinMain() Invalid MAC address read from command line\r\n");
			RETAILMSG(1,(L"ERROR : WinMain() Invalid MAC address read from command line\r\n"));
			return -1;
		}
		printf("Read valid MAC address %s from command line\r\n", buf);
		bAssignMACFromCmdLine = TRUE;	// MAC address is valid, use it
		bAssignMACFromFile = FALSE;		// Don't check for adsload.mac file
	} 
	
	else {	// No parameters were given on the command line, so check for file
		bAssignMACFromCmdLine = FALSE;
	
		nFilenameLen = GetModuleFileName(NULL, wszFilename, BUF_LEN);
		if (!nFilenameLen) {
			RETAILMSG(1,(L"ERROR : WinMain() nFilenameLen == NULL\r\n"));
			return -1;
		}
		pwcTemp = wcsrchr(wszFilename, L'\\');
		if (!pwcTemp) {
			RETAILMSG(1,(L"ERROR : WinMain() pwcTemp == NULL\r\n"));
			return -1;
		}
		pwcTemp++;
		if (!wcscpy(pwcTemp, L"adsload.mac")) {
			RETAILMSG(1,(L"ERROR : WinMain() wcscpy(pwcTemp, L\"adsload.mac\") returned NULL\r\n"));
			return -1;
		}
		fp = _wfopen((const wchar_t *)wszFilename, L"rw+");
		if (fp) {							// MAC file is found
			RETAILMSG(1, (L"\r\nCurrent MAC Address = %02x-%02x-%02x-%02x-%02x-%02x\r\n",
		       temp[0],temp[1],temp[2], temp[3],temp[4],temp[5]));
			wprintf(L"\r\nFound file %s\r\n", wszFilename);
			RETAILMSG(1, (L"Found file %s\r\n", wszFilename));
		
			if (!GetMACFromFile(fp, szNextMac, NextMac))
				return -1;
			bAssignMACFromFile = TRUE;
		} else bAssignMACFromFile = FALSE;	// Didnt find MAC file, will ask for MAC interactively

		if (!bAssignMACFromFile) {
			cIsCorrect = 'N';
			do {
				printf("\r\nEnter new MAC address (format: XX-XX-XX-XX-XX-XX):\r\n");
				scanf("%s", szNextMac);
				printf("You entered: %s\n", szNextMac);
				if (GetMACFromString(szNextMac, NextMac)) {
					printf("Is this correct? Type 'y' for yes or any other key for no: ");
					scanf("%s", &cIsCorrect);
				} else {
					printf("This is an invalid MAC address.  Please try again.\r\n");
				}
			} while (cIsCorrect != 'Y' && cIsCorrect != 'y');		   
	
			printf("\r\nType 'W' to write this MAC address to the chip,\r\nor any other key to quit without writing: ");
			scanf("%s", &cDoWrite);
			}
		}
	
	if(cDoWrite == 'w' || cDoWrite == 'W' || bAssignMACFromFile || bAssignMACFromCmdLine)
    {

		WriteMACAddress(NextMac);		
     	// Reload  MAC address from EEPROM to Register
      	
      	WriteReg( CONTROL_REG, RELOAD );
      	Sleep(1);
      	tmp=ReadReg(CONTROL_REG);
      	while(tmp & 0x3) 
        {
	  		tmp=ReadReg(CONTROL_REG);
	  		Sleep(1);
        }

      	ReadMACAddress(temp);
   
      	printf(" \r\nRead back from chip:  MAC Address = %02x-%02x-%02x-%02x-%02x-%02x",
		   temp[0],temp[1],temp[2], temp[3],temp[4],temp[5]);
		if (bAssignMACFromFile)
			RETAILMSG(1, (L" \r\nRead back from chip:  MAC Address = %02x-%02x-%02x-%02x-%02x-%02x",
				temp[0],temp[1],temp[2], temp[3],temp[4],temp[5]));

		if (VerifyMAC(temp, NextMac))
		{
	  		printf("\r\nSuccesfully programmed MAC Address !!!! \r\n");
			if (bAssignMACFromFile) {
				RETAILMSG(1, (L"\r\nSuccesfully programmed MAC Address !!!! \r\n"));
				fseek(fp, 0, SEEK_SET);	// go to beginning of file
				IncrementMAC(szNextMac);
				fwrite(szNextMac, sizeof(szNextMac) - 1, 1, fp);
				fclose(fp);
				printf("Wrote next MAC address %s back to file\r\n", szNextMac);
				MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, (const char *)szNextMac, -1, wbuf, sizeof(wbuf));
				RETAILMSG(1, (L"Wrote next MAC address %s back to file\r\n", wbuf));
			}
		}
      	else
		{
	  		printf("\r\nError writing MAC Address !!!! \r\n");
			if (bAssignMACFromFile)
				RETAILMSG(1, (L"\r\nError writing MAC Address !!!! \r\n"));
		}
    }

	if (!bAssignMACFromFile && !bAssignMACFromCmdLine) {
		printf("Press <ENTER> to quit: ");
		fflush(stdin);
		getchar();
	}
	VirtualFree(pStartMem, PAGE_SIZE, MEM_DECOMMIT); 
	return 0;
}


// void WriteReg(USHORT reg, USHORT data)
//
// Writes the 16-bit value contained in data to the register identified by reg.
//
// Parameters:
// USHORT reg		Register to write to
// USHORT data		Data to write
void WriteReg(USHORT reg, USHORT data) {
	*((USHORT *)((BYTE *)pMem+reg)) = data;
}


// USHORT ReadReg(USHORT reg)
//
// Reads the 16-bit register identified by reg.
//
// Parameters:
// USHORT reg		Register to read
// Return:			Contents of the register.
USHORT ReadReg(USHORT reg) {
	return *((USHORT *)((BYTE *)pMem+reg));
}


// VOID SelectBank(USHORT Value)
//
// Sets the value of the SMSC91C111 BANKSEL register.
// This selects the current register bank to use.
//
// Parameters:
// USHORT Value		Bank to be selected
VOID SelectBank(USHORT Value)
{
	USHORT bank;

	WriteReg(BANKSEL_REG, Value);
	bank = ReadReg(BANKSEL_REG);
	

	if ((bank & 0x000f) != Value)
    {
		RETAILMSG(1,(L" ERROR : SmcSelectBank() - bank select 0x%x \r\n", bank & 0x000f));
    }
}


// void SMCChipReset()
//
// Resets the SMSC91C111 chip
void SMCChipReset()
{

	// Interupt Mask and ACK
   	SelectBank(BANK2);
   	WriteReg(INTERRUPT_REG, 0x0056);
	
	// Software Reset
   	SelectBank(BANK0);
   	WriteReg(RCR_REG,RCR_EPH_RST);

	Sleep(1);

	// RX disable
   	SelectBank(BANK0);
   	WriteReg(RCR_REG,0x0000);


	// TX disable 
   	SelectBank(BANK0);
   	WriteReg(TCR_REG,0x0000);


	//MMU reset 
   	SelectBank(BANK2);
   	WriteReg(MMUCOMMAND_REG,CMD_RESET);

    Sleep(1);

}


// BOOL GetMACFromString(UCHAR *buf, UCHAR *MAC)
//
// Takes an ASCII string representation of a MAC address and converts
// it to a numerical version in a 6-byte array.  The first
// item in the array will be the left-most byte of the MAC address
// string.
//
// Parameters:
// UCHAR *buf	Pointer to ASCII string containing a MAC address in the format:
//				xx-xx-xx-xx-xx-xx
// UCHAR *MAC	Pointer to 6-byte array of UCHARs to hold the result of the
//				conversion.
// Returns:		TRUE if the conversion completed succesfully, FALSE otherwise
BOOL GetMACFromString(UCHAR *buf, UCHAR *MAC) {
	UCHAR *temp1, *temp2;
	int i;
	for (i = 0,	temp1 = buf; i < 5; i++, temp1 = temp2 + 1) {
		temp2 = (unsigned char *)strpbrk((const char *)temp1, "-");
		if ((temp2 - temp1) != 2)
			return FALSE;	// number is not 2 digits
		if (!GetHexDigit(temp1, &MAC[i]))	// Check that the number is a valid hex
			return FALSE;					// number, and convert it from ASCII
	}
	if (!GetHexDigit(temp1, &MAC[5]))	// Check and convert the last set of digits, 
		return FALSE;					// since not terminated with '-'
	//if (*(temp1 + 2) != '\0')	// Check that this is the end of the string
	//	return FALSE;
	return TRUE;		// Conversion complete with all checks passed
}


// BOOL GetHexDigit(UCHAR *src, UCHAR *dest)
//
// Takes an ASCII string representation of a 2-digit hexadecimal number and converts
// to a single-byte numerical version.
//
// Parameters:
// UCHAR *src	Pointer to a string containing a 2-digit ASCII representation of
//				an 8 byte hexadecimal number.
// UCHAR *dest	Pointer to a byte that will receive the result of the
//				conversion.
// Returns:		TRUE if the conversion completed succesfully, FALSE otherwise
BOOL GetHexDigit(UCHAR *src, UCHAR *dest) {
	int i;
	UCHAR tmp[2];
	for (i = 0; i < 2; i++, src++) {		// Check each digit
		if (*src >= '0' && *src <= '9') {	// It can be between '0' and '9',
			tmp[i] = *src - '0';
		}
		else {
			if (*src >= 'A' && *src <= 'F') {	// or 'A' and 'F',
				tmp[i] = *src - 'A' + 10;
			} else {
				if (*src >= 'a' && *src <= 'f') { // or 'a' and 'f'.
					tmp[i] = *src - 'a' + 10;
				} else {
					return FALSE;			// Otherwise it's no good.
				}
			}
		}
	}
	*dest = (tmp[0] << 4) | tmp[1];			// The first character is the high order nibble
	return TRUE;							// and the second is the low order nibble.
}


// void IncrementMAC(UCHAR *buf)
//
// Takes an ASCII string representation of a MAC address and increments it by one.
// Example: 00-60-0c-00-cc-50 will be incremented to 00-60-0c-00-cc-51
//
// Parameters:
// UCHAR *buf	Pointer to ASCII string containing a MAC address in the format:
//				xx-xx-xx-xx-xx-xx.  This string will be overwritten by the
//				incremented string.
void IncrementMAC(UCHAR *buf) {
	UCHAR MAC [6];
	LONGLONG llMAC = 0;
	int i;

	GetMACFromString(buf, MAC);
	for (i = 0; i < 6; i++) {
		llMAC |= MAC[i];
		llMAC <<= 8;
	}
	llMAC >>=8;
	llMAC++;
	for (i = 5; i > -1; i--) {
		MAC[i] = (UCHAR)(llMAC & 0xFF);
		llMAC >>= 8;
	}
	sprintf((char *)buf, "%02x-%02x-%02x-%02x-%02x-%02x", MAC[0], MAC[1], MAC[2], MAC[3], MAC[4], MAC[5]);
}


// BOOL IsMACInRange(UCHAR *MiddleMac, UCHAR *LowMac, UCHAR *HighMac)
//
// Takes ASCII string representations of three MAC addresses: MiddleMac,
// LowMac, and HighMac.  Determines if MiddleMac is between LowMac and HighMac,
// inclusive.
//
// Parameters:
// UCHAR *MiddleMac		Pointer to ASCII string containing a MAC address in the format:
//						xx-xx-xx-xx-xx-xx.  This is the MAC that should be between
//						the other two.
// UCHAR *LowMac		Lower bound MAC address.
// UCHAR *HighMac		Upper bound MAC address.
// Returns:				TRUE if MiddleMac is between LowMac and HighMac, otherwise FALSE.
BOOL IsMACInRange(UCHAR *MiddleMac, UCHAR *LowMac, UCHAR *HighMac) {
	BOOL bRet;	
	LONGLONG llMiddleMac, llLowMac, llHighMac;
	int i;
	
	for (i = 0, llMiddleMac = 0; i < 6; i++) {
		llMiddleMac <<= 8;
		llMiddleMac |= MiddleMac[i];
	}
	
	for (i = 0, llLowMac = 0; i < 6; i++) {
		llLowMac <<= 8;
		llLowMac |= LowMac[i];
	}

	for (i = 0, llHighMac = 0; i < 6; i++) {
		llHighMac <<= 8;
		llHighMac |= HighMac[i];
	}
	
	if (llMiddleMac > llHighMac || llMiddleMac < llLowMac)
		bRet = FALSE;
	else
		bRet = TRUE;

	return bRet;
}


// void ReadMACAddress(UCHAR *dest)
//
// Reads the MAC address from the SMSC91C111 and stores it in a 6 byte array.
// Example: For the address 11-22-33-44-55-66, dest[0] will hold 0x11, dest[1]
// will hold 0x22, etc.
//
// Parameters:
// UCHAR *dest	Pointer to 6-byte array of UCHARs that will receive the MAC
//				address.
void ReadMACAddress(UCHAR *dest) {
	USHORT MacAddr[3];
	
	SelectBank(BANK1);
  	MacAddr[0] = ReadReg(MACADDR_REG0);
  	dest[0] = (UCHAR)MacAddr[0];
  	dest[1] = (UCHAR)(MacAddr[0] >> 8);
  	MacAddr[1] = ReadReg(MACADDR_REG1);
  	dest[2] = (UCHAR)MacAddr[1];
  	dest[3] = (UCHAR)(MacAddr[1] >> 8);
  	MacAddr[2] = ReadReg(MACADDR_REG2);
  	dest[4] = (UCHAR)MacAddr[2];
  	dest[5] = (UCHAR)(MacAddr[2] >> 8);
}


// void WriteMACAddress(UCHAR *src) 
//
// Writes the MAC address contained in the 6 byte array referenced by src 
// to the SMSC91C111.
// Example: To write the address 11-22-33-44-55-66, src[0] should contain
// 0x11, src[1] should contain 0x22, etc.
//
// Parameters:
// UCHAR *src	Pointer to 6-byte array of UCHARs that contains the MAC
//				address to write.
void WriteMACAddress(UCHAR *src) {
	USHORT MacAddr [3];	
	USHORT tmp;

	MacAddr[0] = src[1];
	MacAddr[0] <<= 8;
	MacAddr[0] |= src[0];
	MacAddr[1] = src[3];
	MacAddr[1] <<= 8;
	MacAddr[1] |= src[2];
	MacAddr[2] = src[5];
	MacAddr[2] <<= 8;
	MacAddr[2] |= src[4];
	
	printf("\r\nWriting MAC address to chip \r\n ==>\r\n");
	RETAILMSG(1, (L"\r\nWriting MAC address to chip \r\n ==>\r\n"));
	SelectBank(BANK2 );
	WriteReg(POINTER_REG,0x20);
	
	SelectBank(BANK1 );
	
	WriteReg(GENERAL_REG,MacAddr[0]);
	WriteReg(CONTROL_REG, (ROM_SEL) );
	WriteReg(CONTROL_REG, (ROM_SEL| STORE) );
	Sleep(1);
	tmp=ReadReg(CONTROL_REG);
	
	while(tmp & 0x3) 
	{
		tmp=ReadReg(CONTROL_REG);
		Sleep(1);

	}

	SelectBank(BANK2 );
	WriteReg(POINTER_REG,0x21);
	
	SelectBank(BANK1 );
	
	WriteReg( GENERAL_REG,MacAddr[1]);
	
	WriteReg( CONTROL_REG, (ROM_SEL| STORE));
	Sleep(1);
	tmp=ReadReg(CONTROL_REG);
	while(tmp & 0x3) 
	{
		tmp=ReadReg(CONTROL_REG);

		Sleep(1);
	}
	
	SelectBank(BANK2 );
	WriteReg(POINTER_REG,0x22);
	
	SelectBank(BANK1 );
	
	WriteReg( GENERAL_REG,MacAddr[2]);
	
	WriteReg( CONTROL_REG, (ROM_SEL| STORE) );	
	Sleep(1);
	tmp=ReadReg(CONTROL_REG);

	while(tmp & 0x3) 
	{
		tmp=ReadReg(CONTROL_REG);
		Sleep(1);
	}
}


// BOOL VerifyMAC(UCHAR *src1, UCHAR *src2)
//
// Compares two MAC addresses contained in the 6 byte arrays referenced by src1 and src2.
//
// Parameters:
// UCHAR *src1	Pointer to MAC address
// UCHAR *src2	Pointer to MAC address
// Return:		TRUE if the MAC addresses are equal, FALSE otherwise.
BOOL VerifyMAC(UCHAR *src1, UCHAR *src2) {
	if (src1[0] == src2[0] &&  
		src1[1] == src2[1] && 
		src1[2] == src2[2] && 
		src1[3] == src2[3] && 
		src1[4] == src2[4] && 
		src1[5] == src2[5])
		return TRUE;
	else return FALSE;
}

// BOOL GetMACFromFile(FILE *fp, UCHAR *szNextMac, UCHAR *NextMac)
//
// Compares two MAC addresses contained in the 6 byte arrays referenced by src1 and src2.
//
// Parameters:
// FILE *fp				File pointer to ADSLOAD.MAC
// UCHAR *szNextMac		Pointer to string where the ASCII representation of the next MAC address
//						will be stored.
// UCHAR *NextMac		Pointer to 6-byte array where the numerical MAC address will be stored.
// Return:				TRUE if success, FALSE if there were errors.
BOOL GetMACFromFile(FILE *fp, UCHAR *szNextMac, UCHAR *NextMac) {
	UCHAR szStartMac [] = "xx-xx-xx-xx-xx-xx";
	UCHAR StartMac [6];
	UCHAR szEndMac [] = "xx-xx-xx-xx-xx-xx";
	UCHAR EndMac [6];
	UCHAR temp [6];
	UCHAR buf [128];
	WCHAR wbuf [] = L"xx-xx-xx-xx-xx-xx";

	fgets((char *)buf, sizeof(buf), fp);
	strcpy((char *)szNextMac, (const char *)buf);
	if (!GetMACFromString(szNextMac, NextMac)) {
		RETAILMSG(1,(L"ERROR : GetMACFromFile() Invalid next MAC address read from file\r\n"));
		return FALSE;
	}
	
	fgets((char *)buf, sizeof(buf), fp);
	strcpy((char *)szStartMac, (const char *)buf);
	if (!GetMACFromString(szStartMac, StartMac)) {
		RETAILMSG(1,(L"ERROR : GetMACFromFile() Invalid start MAC address read from file\r\n"));
		return FALSE;
	}
	fgets((char *)buf, sizeof(buf), fp);
	strcpy((char *)szEndMac, (const char *)buf);
	if (!GetMACFromString(szEndMac, EndMac)) {
		RETAILMSG(1,(L"ERROR : GetMACFromFile() Invalid ending MAC address read from file\r\n"));
		return FALSE;
	}

	// Check that MAC address constraints make sense
	strcpy((char *)buf, "FF-FF-FF-FF-FF-FF");
	GetMACFromString(buf, temp);
	if (!IsMACInRange((UCHAR *)EndMac, (UCHAR *)StartMac, (UCHAR *)temp)) {
		RETAILMSG(1,(L"ERROR: WinMain() Ending MAC address less than starting MAC address.\r\n"));
		return FALSE;
	}		

	// Check that the next MAC address to use is within the constraints
	if (!IsMACInRange((UCHAR *)NextMac, (UCHAR *)StartMac, (UCHAR *)EndMac)) {
		RETAILMSG(1,(L"ERROR: WinMain() MAC address out of range.\r\n"));
		return FALSE;
	}
	
	printf("Read valid MAC address %s from file\r\n", szNextMac);
	MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, (const char *)szNextMac, -1, wbuf, sizeof(wbuf));
	RETAILMSG(1, (L"Read valid MAC address %s from file\r\n", wbuf));
	return TRUE;
}




