/*++

Copyright:

    Copyright (c) 1998, 1999 BSQUARE Corporation

Module Name:

    smchw.h

Abstract:

    This header file is used to provide the low level definitions for the 
    SMC9000 Ethernet driver.

--*/

#ifndef _SMCHW_H
#define _SMCHW_H

// Chip number definitions.
//

#define CHIP_91C90          3 
#define CHIP_91C94          4  
#define CHIP_91C95          5 
#define CHIP_91C100         7
#define CHIP_91C111         9

// mapping to ThinClient registers
//#define TC_ADDRESS(n) ((n>>1)<<3)
#define TC_ADDRESS(n) 	(n)
//
// The bank select register is the same for all 4 banks.
//

#define BANKSEL_REG     TC_ADDRESS(14)

//
// Bank 0 registers.
//

#define TCR_REG         TC_ADDRESS(0)
#define EPHSTATUS_REG   TC_ADDRESS(2)
#define RCR_REG         TC_ADDRESS(4)
#define COUNTER_REG     TC_ADDRESS(6)
#define MIR_REG         TC_ADDRESS(8)
#define MCR_REG         TC_ADDRESS(10)

//
// Bank 1 registers.
//

#define CONFIG_REG      TC_ADDRESS(0)
#define BASE_REG        TC_ADDRESS(2)
#define MACADDR_REG0    TC_ADDRESS(4)
#define MACADDR_REG1    TC_ADDRESS(6)
#define MACADDR_REG2    TC_ADDRESS(8)
#define GENERAL_REG     TC_ADDRESS(10)
#define CONTROL_REG     TC_ADDRESS(12)

//
// Bank 2 registers.
//

#define MMUCOMMAND_REG      TC_ADDRESS(0)
#define PNR_ARR_REG         TC_ADDRESS(2)
#define FIFOPORTS_REG       TC_ADDRESS(4)
#define POINTER_REG         TC_ADDRESS(6)
#define DATA_REG            TC_ADDRESS(8)
#define AUXDATA_REG         TC_ADDRESS(10)
#define INTERRUPT_REG       TC_ADDRESS(12)
#define INTERRUPT_MASK_REG  TC_ADDRESS(13)  

//
// Bank 3 registers.
//

#define MULTITAB_REG0   TC_ADDRESS(0)
#define MULTITAB_REG1   TC_ADDRESS(2)
#define MULTITAB_REG2   TC_ADDRESS(4)
#define MULTITAB_REG3   TC_ADDRESS(6)
#define MGMT_REG        TC_ADDRESS(8)
#define REVISION_REG    TC_ADDRESS(10)
#define ERCV_REG        TC_ADDRESS(12)

//
// These values are written to the bank select register to change banks.
//

#define BANK0 0x0
#define BANK1 0x1
#define BANK2 0x2
#define BANK3 0x3

//
// Control reg defs (used for CONTROL_REG of bank 1).
//
#define STORE		            0x0001  
#define RELOAD		            0x0002  
#define ROM_SEL		            0x0004  



//
// Config reg defs (used for CONFIG_REG of bank 1).
//

#define CFG_NO_WAIT             0x1000  //  No wait states
#define CFG_FULL_STEP           0x0400  //  Full step signalling to AUI
#define CFG_SETSQLCH            0x0200  //  Set squelch level to 240mV
#define CFG_AUI_SEL             0x0100  //  =1 AUI, =0 twisted pair transeiver
#define CFG_16BIT               0x0080  //  System bus width
#define CFG_DIS_LINK            0x0040  //  Disable link test functions (10BASET)
#define CFG_RAM_SZ              0x0020  //  =0 8kb, =1 32kb
#define CFG_RAM_WDTH            0x0010  //  =0 8 bit, =1 16 bit
#define CFG_FAST_RAM            0x0008  //  SRAM timing
#define CFG_MASK_IRQ            0x0002  //  IRQ used =0  IRQ9
                                        //           =1  IRQ3
                                        //           =2  IRQA
                                        //           =3  IRQB

//
// EPH Status reg values - used for EPHSTATUS_REG of bank 0.
//

#define EPH_UNDERRUN            0x8000  //  Frame uderrun
#define EPH_LINKERROR           0x4000  //  10BASET link error condition
#define EPH_RXOVERRUN           0x2000  //  Receiver overrun
#define EPH_COUNTER             0x1000  //  Counter roll over
#define EPH_EXDEFER             0x0800  //  Excessive deferral
#define EPH_CARRIER             0x0400  //  Carrier not present
#define EPH_LATE                0x0200  //  Late collision
#define EPH_DEFER               0x0080  //  Frame was deferred
#define EPH_BCAST               0x0040  //  Last frame was broadcast
#define EPH_SQET                0x0020  //  Signal Quality Error
#define EPH_16COL               0x0010  //  Too many collisions
#define EPH_MCAST               0x0008  //  Last frame was multicast
#define EPH_MULTICOL            0x0004  //  Multiple collisions on last frame
#define EPH_1COL                0x0002  //  Single collision on last frame
#define EPH_TX_OK               0x0001  //  Frame successfully transmitted

//
// MMU Commands (used for MMUCOMMAND_REG of bank 2).
//

#define CMD_NOP                 0       //  No-Op command
#define CMD_ALLOC               0x0020  //  Allocate memory
#define CMD_RESET               0x0040  //  Reset MMU to initial state
#define CMD_REM_TOP             0x0060  //  Remove frame from top of RX fifo
#define CMD_REM_REL_TOP         0x0080  //  Remove and release top of RX fifo
#define CMD_REL_SPEC            0x00a0  //  Release specific packet
#define CMD_ENQ_TX              0x00c0  //  Enqueue to xmit fifo
#define CMD_ENQ_RX              0x00e0  //  Reset xmit fifos (should only be done
                                        //  with transmitter disabled)
#define MMU_CMD_BUSY            0x0001  //  MMU busy, don't modify PNR

//
// Allocation Result Register (low byte of PNR_ARR_REG of bank 2).
//

#define ARR_FAIL                0x0080  //  Allocation failed
#define ARR_ALLOC_MSK           0x007f  //  Mask allocated packet number
                                        //  Actually 1fh but use full mask for
                                        //  upward compatibility.

//
// Pointer Register (POINTER_REG of bank 2).
//

#define PTR_RCV                 0x8000  //  Access is to receive area
#define PTR_AUTO                0x4000  //  Auto-increment on access
#define PTR_READ                0x2000  //  =1 then read operation
                                        //  =0 then write operation
#define PTR_ETEN                0x1000  //  Detect early transmit underrun
#define PTR_NOT_EMPTY           0x0800  //  Write fifo not empty 
#define PTR_OFFSET              0x03ff  //  Mask pointer value

//
// Fifo Ports Register (FIFOPORTS_REG of bank 2).
//

#define FIFO_EMPTY              0x80    //  No packet at top of fifo
#define FIFO_TX_PKT_MASK        0x7f    //  Mask top packet number
                                        //  Actually 1fh but use full mask for
                                        //  upward compatibility.

//
// Interrupt flags that can be set/acknowledged (INTERRUPT_REG of bank 2).
//

#define ERCV_INT        0x0040
#define EPH_INT         0x0020
#define RX_OVRN_INT     0x0010
#define ALLOC_INT       0x0008
#define TXEMPTY_INT     0x0004
#define TX_INT          0x0002
#define RCV_INT         0x0001

//
// Interrupt mask bits, if these are set the interrupt is enabled (INTERRUPT_MASK_REG of bank 2).
//

#define ERCV_INTM       0x4000
#define EPH_INTM        0x2000
#define RX_OVRN_INTM    0x1000
#define ALLOC_INTM      0x0800
#define TXEMPTY_INTM    0x0400
#define TX_INTM         0x0200
#define RCV_INTM        0x0100

//
// Register values used for initialization.
//

#define CONTROL_REG_INIT    0x0920
#define TCR_REG_INIT        0x0081
#define MCR_REG_INIT        0x0006
#define RCR_REG_INIT        0x0100

/* Recieve control register */

#define RCR_EPH_RST				0x8000	/* software activated reset */
#define RCR_FILT_CAR			0x4000	/* filter carrier sense 12 bits */           	
#define	RCR_STRIP_CRC			0x0200	/* when set, strips CRC */
#define RCR_RXEN				0x0100	/* ENABLE reciever */
#define RCR_ALMUL				0x0004	/* receive all multicast packets */
#define	RCR_PRMS				0x0002	/* enable promiscuous mode */
#define RCR_RX_ABORT			0x0001	/* set if reciever overrun */          
#define RCR_RESET				0x0	/* set it to a base state */

//
// The following definitions are used to check for error in receivered frame.
//

#define FRAME_FILTER        0xAC00
#define BROADCAST_FILTER    0x4000
#define MULTCAST_FILTER     0x0001

#endif  // _SMCHW_H